package com.sun.media.sound;

import java.util.Iterator;
import java.util.Map;
import java.util.HashMap;
import javax.sound.midi.Patch;
import java.util.ArrayList;
import java.util.List;

public final class SF2Instrument extends ModelInstrument
{
    String name;
    int preset;
    int bank;
    long library;
    long genre;
    long morphology;
    SF2GlobalRegion globalregion;
    List<SF2InstrumentRegion> regions;
    
    public SF2Instrument() {
        super(null, null, null, null);
        this.name = "";
        this.preset = 0;
        this.bank = 0;
        this.library = 0L;
        this.genre = 0L;
        this.morphology = 0L;
        this.globalregion = null;
        this.regions = new ArrayList();
    }
    
    public SF2Instrument(SF2Soundbank sf2Soundbank) {
        super(sf2Soundbank, null, null, null);
        this.name = "";
        this.preset = 0;
        this.bank = 0;
        this.library = 0L;
        this.genre = 0L;
        this.morphology = 0L;
        this.globalregion = null;
        this.regions = new ArrayList();
    }
    
    @Override
    public String getName() {
        return this.name;
    }
    
    public void setName(String name) {
        this.name = name;
    }
    
    @Override
    public Patch getPatch() {
        if (this.bank == 128) {
            return new ModelPatch(0, this.preset, true);
        }
        return new ModelPatch(this.bank << 7, this.preset, false);
    }
    
    public void setPatch(Patch patch) {
        if (patch instanceof ModelPatch && ((ModelPatch)patch).isPercussion()) {
            this.bank = 128;
            this.preset = patch.getProgram();
        }
        else {
            this.bank = patch.getBank() >> 7;
            this.preset = patch.getProgram();
        }
    }
    
    @Override
    public Object getData() {
        return null;
    }
    
    public long getGenre() {
        return this.genre;
    }
    
    public void setGenre(long genre) {
        this.genre = genre;
    }
    
    public long getLibrary() {
        return this.library;
    }
    
    public void setLibrary(long library) {
        this.library = library;
    }
    
    public long getMorphology() {
        return this.morphology;
    }
    
    public void setMorphology(long morphology) {
        this.morphology = morphology;
    }
    
    public List<SF2InstrumentRegion> getRegions() {
        return this.regions;
    }
    
    public SF2GlobalRegion getGlobalRegion() {
        return this.globalregion;
    }
    
    public void setGlobalZone(SF2GlobalRegion globalregion) {
        this.globalregion = globalregion;
    }
    
    @Override
    public String toString() {
        if (this.bank == 128) {
            return "Drumkit: " + this.name + " preset #" + this.preset;
        }
        return "Instrument: " + this.name + " bank #" + this.bank + " preset #" + this.preset;
    }
    
    @Override
    public ModelPerformer[] getPerformers() {
        int n = 0;
        Iterator<SF2InstrumentRegion> iterator = this.regions.iterator();
        while (iterator.hasNext()) {
            n += ((SF2InstrumentRegion)iterator.next()).getLayer().getRegions().size();
        }
        ModelPerformer[] array = new ModelPerformer[n];
        int n2 = 0;
        SF2GlobalRegion globalregion = this.globalregion;
        for (SF2InstrumentRegion sf2InstrumentRegion : this.regions) {
            HashMap o = new HashMap();
            o.putAll(sf2InstrumentRegion.getGenerators());
            if (globalregion != null) {
                o.putAll(globalregion.getGenerators());
            }
            SF2Layer layer = sf2InstrumentRegion.getLayer();
            SF2GlobalRegion globalRegion = layer.getGlobalRegion();
            for (SF2LayerRegion sf2LayerRegion : layer.getRegions()) {
                ModelPerformer o2 = new ModelPerformer();
                if (sf2LayerRegion.getSample() != null) {
                    o2.setName(sf2LayerRegion.getSample().getName());
                }
                else {
                    o2.setName(layer.getName());
                }
                array[n2++] = o2;
                int keyFrom = 0;
                int keyTo = 127;
                int velFrom = 0;
                int velTo = 127;
                if (sf2LayerRegion.contains(57)) {
                    o2.setExclusiveClass(sf2LayerRegion.getInteger(57));
                }
                if (sf2LayerRegion.contains(43)) {
                    byte[] bytes = sf2LayerRegion.getBytes(43);
                    if (bytes[0] >= 0 && bytes[0] > keyFrom) {
                        keyFrom = bytes[0];
                    }
                    if (bytes[1] >= 0 && bytes[1] < keyTo) {
                        keyTo = bytes[1];
                    }
                }
                if (sf2LayerRegion.contains(44)) {
                    byte[] bytes2 = sf2LayerRegion.getBytes(44);
                    if (bytes2[0] >= 0 && bytes2[0] > velFrom) {
                        velFrom = bytes2[0];
                    }
                    if (bytes2[1] >= 0 && bytes2[1] < velTo) {
                        velTo = bytes2[1];
                    }
                }
                if (sf2InstrumentRegion.contains(43)) {
                    byte[] bytes3 = sf2InstrumentRegion.getBytes(43);
                    if (bytes3[0] > keyFrom) {
                        keyFrom = bytes3[0];
                    }
                    if (bytes3[1] < keyTo) {
                        keyTo = bytes3[1];
                    }
                }
                if (sf2InstrumentRegion.contains(44)) {
                    byte[] bytes4 = sf2InstrumentRegion.getBytes(44);
                    if (bytes4[0] > velFrom) {
                        velFrom = bytes4[0];
                    }
                    if (bytes4[1] < velTo) {
                        velTo = bytes4[1];
                    }
                }
                o2.setKeyFrom(keyFrom);
                o2.setKeyTo(keyTo);
                o2.setVelFrom(velFrom);
                o2.setVelTo(velTo);
                int short1 = sf2LayerRegion.getShort(0);
                int short2 = sf2LayerRegion.getShort(1);
                int short3 = sf2LayerRegion.getShort(2);
                int short4 = sf2LayerRegion.getShort(3);
                int n3 = short1 + sf2LayerRegion.getShort(4) * 32768;
                int n4 = short2 + sf2LayerRegion.getShort(12) * 32768;
                int n5 = short3 + sf2LayerRegion.getShort(45) * 32768;
                int n6 = short4 + sf2LayerRegion.getShort(50) * 32768;
                int n7 = n5 - n3;
                int n8 = n6 - n3;
                SF2Sample sample = sf2LayerRegion.getSample();
                int n9 = sample.originalPitch;
                if (sf2LayerRegion.getShort(58) != -1) {
                    n9 = sf2LayerRegion.getShort(58);
                }
                float n10 = (float)(-n9 * 100 + sample.pitchCorrection);
                ModelByteBuffer modelByteBuffer = sample.getDataBuffer();
                ModelByteBuffer modelByteBuffer2 = sample.getData24Buffer();
                if (n3 != 0 || n4 != 0) {
                    modelByteBuffer = modelByteBuffer.subbuffer((long)(n3 * 2), modelByteBuffer.capacity() + (long)(n4 * 2));
                    if (modelByteBuffer2 != null) {
                        modelByteBuffer2 = modelByteBuffer2.subbuffer((long)n3, modelByteBuffer2.capacity() + (long)n4);
                    }
                }
                ModelByteBufferWavetable o3 = new ModelByteBufferWavetable(modelByteBuffer, sample.getFormat(), n10);
                if (modelByteBuffer2 != null) {
                    o3.set8BitExtensionBuffer(modelByteBuffer2);
                }
                HashMap o4 = new HashMap();
                if (globalRegion != null) {
                    o4.putAll(globalRegion.getGenerators());
                }
                o4.putAll(sf2LayerRegion.getGenerators());
                for (Map.Entry entry : o.entrySet()) {
                    int n11;
                    if (!o4.containsKey(entry.getKey())) {
                        n11 = sf2LayerRegion.getShort((int)entry.getKey());
                    }
                    else {
                        n11 = (short)o4.get(entry.getKey());
                    }
                    o4.put(entry.getKey(), Short.valueOf((short)(n11 + (short)entry.getValue())));
                }
                int generatorValue = this.getGeneratorValue(o4, 54);
                if ((generatorValue == 1 || generatorValue == 3) && sample.startLoop >= 0L && sample.endLoop > 0L) {
                    o3.setLoopStart((float)(int)(sample.startLoop + (long)n7));
                    o3.setLoopLength((float)(int)(sample.endLoop - sample.startLoop + (long)n8 - (long)n7));
                    if (generatorValue == 1) {
                        o3.setLoopType(1);
                    }
                    if (generatorValue == 3) {
                        o3.setLoopType(2);
                    }
                }
                o2.getOscillators().add(o3);
                int generatorValue2 = this.getGeneratorValue(o4, 33);
                int generatorValue3 = this.getGeneratorValue(o4, 34);
                int generatorValue4 = this.getGeneratorValue(o4, 35);
                int generatorValue5 = this.getGeneratorValue(o4, 36);
                int generatorValue6 = this.getGeneratorValue(o4, 37);
                int generatorValue7 = this.getGeneratorValue(o4, 38);
                if (generatorValue4 != -12000) {
                    int generatorValue8 = this.getGeneratorValue(o4, 39);
                    generatorValue4 = (short)(generatorValue4 + 60 * generatorValue8);
                    o2.getConnectionBlocks().add(new ModelConnectionBlock(new ModelSource(ModelSource.SOURCE_NOTEON_KEYNUMBER), (double)(float)(-generatorValue8 * 128), new ModelDestination(ModelDestination.DESTINATION_EG1_HOLD)));
                }
                if (generatorValue5 != -12000) {
                    int generatorValue9 = this.getGeneratorValue(o4, 40);
                    generatorValue5 = (short)(generatorValue5 + 60 * generatorValue9);
                    o2.getConnectionBlocks().add(new ModelConnectionBlock(new ModelSource(ModelSource.SOURCE_NOTEON_KEYNUMBER), (double)(float)(-generatorValue9 * 128), new ModelDestination(ModelDestination.DESTINATION_EG1_DECAY)));
                }
                this.addTimecentValue(o2, ModelDestination.DESTINATION_EG1_DELAY, (short)generatorValue2);
                this.addTimecentValue(o2, ModelDestination.DESTINATION_EG1_ATTACK, (short)generatorValue3);
                this.addTimecentValue(o2, ModelDestination.DESTINATION_EG1_HOLD, (short)generatorValue4);
                this.addTimecentValue(o2, ModelDestination.DESTINATION_EG1_DECAY, (short)generatorValue5);
                int n12 = (short)(1000 - generatorValue6);
                if (n12 < 0) {
                    n12 = 0;
                }
                if (n12 > 1000) {
                    n12 = 1000;
                }
                this.addValue(o2, ModelDestination.DESTINATION_EG1_SUSTAIN, (short)n12);
                this.addTimecentValue(o2, ModelDestination.DESTINATION_EG1_RELEASE, (short)generatorValue7);
                if (this.getGeneratorValue(o4, 11) != 0 || this.getGeneratorValue(o4, 7) != 0) {
                    int generatorValue10 = this.getGeneratorValue(o4, 25);
                    int generatorValue11 = this.getGeneratorValue(o4, 26);
                    int generatorValue12 = this.getGeneratorValue(o4, 27);
                    int generatorValue13 = this.getGeneratorValue(o4, 28);
                    int generatorValue14 = this.getGeneratorValue(o4, 29);
                    int generatorValue15 = this.getGeneratorValue(o4, 30);
                    if (generatorValue12 != -12000) {
                        int generatorValue16 = this.getGeneratorValue(o4, 31);
                        generatorValue12 = (short)(generatorValue12 + 60 * generatorValue16);
                        o2.getConnectionBlocks().add(new ModelConnectionBlock(new ModelSource(ModelSource.SOURCE_NOTEON_KEYNUMBER), (double)(float)(-generatorValue16 * 128), new ModelDestination(ModelDestination.DESTINATION_EG2_HOLD)));
                    }
                    if (generatorValue13 != -12000) {
                        int generatorValue17 = this.getGeneratorValue(o4, 32);
                        generatorValue13 = (short)(generatorValue13 + 60 * generatorValue17);
                        o2.getConnectionBlocks().add(new ModelConnectionBlock(new ModelSource(ModelSource.SOURCE_NOTEON_KEYNUMBER), (double)(float)(-generatorValue17 * 128), new ModelDestination(ModelDestination.DESTINATION_EG2_DECAY)));
                    }
                    this.addTimecentValue(o2, ModelDestination.DESTINATION_EG2_DELAY, (short)generatorValue10);
                    this.addTimecentValue(o2, ModelDestination.DESTINATION_EG2_ATTACK, (short)generatorValue11);
                    this.addTimecentValue(o2, ModelDestination.DESTINATION_EG2_HOLD, (short)generatorValue12);
                    this.addTimecentValue(o2, ModelDestination.DESTINATION_EG2_DECAY, (short)generatorValue13);
                    if (generatorValue14 < 0) {
                        generatorValue14 = 0;
                    }
                    if (generatorValue14 > 1000) {
                        generatorValue14 = 1000;
                    }
                    this.addValue(o2, ModelDestination.DESTINATION_EG2_SUSTAIN, (double)(1000 - generatorValue14));
                    this.addTimecentValue(o2, ModelDestination.DESTINATION_EG2_RELEASE, (short)generatorValue15);
                    if (this.getGeneratorValue(o4, 11) != 0) {
                        o2.getConnectionBlocks().add(new ModelConnectionBlock(new ModelSource(ModelSource.SOURCE_EG2), (double)this.getGeneratorValue(o4, 11), new ModelDestination(ModelDestination.DESTINATION_FILTER_FREQ)));
                    }
                    if (this.getGeneratorValue(o4, 7) != 0) {
                        o2.getConnectionBlocks().add(new ModelConnectionBlock(new ModelSource(ModelSource.SOURCE_EG2), (double)this.getGeneratorValue(o4, 7), new ModelDestination(ModelDestination.DESTINATION_PITCH)));
                    }
                }
                if (this.getGeneratorValue(o4, 10) != 0 || this.getGeneratorValue(o4, 5) != 0 || this.getGeneratorValue(o4, 13) != 0) {
                    int generatorValue18 = this.getGeneratorValue(o4, 22);
                    this.addTimecentValue(o2, ModelDestination.DESTINATION_LFO1_DELAY, this.getGeneratorValue(o4, 21));
                    this.addValue(o2, ModelDestination.DESTINATION_LFO1_FREQ, (short)generatorValue18);
                }
                int generatorValue19 = this.getGeneratorValue(o4, 24);
                this.addTimecentValue(o2, ModelDestination.DESTINATION_LFO2_DELAY, this.getGeneratorValue(o4, 23));
                this.addValue(o2, ModelDestination.DESTINATION_LFO2_FREQ, (short)generatorValue19);
                if (this.getGeneratorValue(o4, 6) != 0) {
                    o2.getConnectionBlocks().add(new ModelConnectionBlock(new ModelSource(ModelSource.SOURCE_LFO2, false, true), (double)this.getGeneratorValue(o4, 6), new ModelDestination(ModelDestination.DESTINATION_PITCH)));
                }
                if (this.getGeneratorValue(o4, 10) != 0) {
                    o2.getConnectionBlocks().add(new ModelConnectionBlock(new ModelSource(ModelSource.SOURCE_LFO1, false, true), (double)this.getGeneratorValue(o4, 10), new ModelDestination(ModelDestination.DESTINATION_FILTER_FREQ)));
                }
                if (this.getGeneratorValue(o4, 5) != 0) {
                    o2.getConnectionBlocks().add(new ModelConnectionBlock(new ModelSource(ModelSource.SOURCE_LFO1, false, true), (double)this.getGeneratorValue(o4, 5), new ModelDestination(ModelDestination.DESTINATION_PITCH)));
                }
                if (this.getGeneratorValue(o4, 13) != 0) {
                    o2.getConnectionBlocks().add(new ModelConnectionBlock(new ModelSource(ModelSource.SOURCE_LFO1, false, true), (double)this.getGeneratorValue(o4, 13), new ModelDestination(ModelDestination.DESTINATION_GAIN)));
                }
                if (sf2LayerRegion.getShort(46) != -1) {
                    this.addValue(o2, ModelDestination.DESTINATION_KEYNUMBER, (double)sf2LayerRegion.getShort(46) / 128.0);
                }
                if (sf2LayerRegion.getShort(47) != -1) {
                    this.addValue(o2, ModelDestination.DESTINATION_VELOCITY, (double)sf2LayerRegion.getShort(47) / 128.0);
                }
                if (this.getGeneratorValue(o4, 8) < 13500) {
                    int generatorValue20 = this.getGeneratorValue(o4, 8);
                    int generatorValue21 = this.getGeneratorValue(o4, 9);
                    this.addValue(o2, ModelDestination.DESTINATION_FILTER_FREQ, (short)generatorValue20);
                    this.addValue(o2, ModelDestination.DESTINATION_FILTER_Q, (short)generatorValue21);
                }
                int n13 = 100 * this.getGeneratorValue(o4, 51) + this.getGeneratorValue(o4, 52);
                if (n13 != 0) {
                    this.addValue(o2, ModelDestination.DESTINATION_PITCH, (short)n13);
                }
                if (this.getGeneratorValue(o4, 17) != 0) {
                    this.addValue(o2, ModelDestination.DESTINATION_PAN, this.getGeneratorValue(o4, 17));
                }
                if (this.getGeneratorValue(o4, 48) != 0) {
                    this.addValue(o2, ModelDestination.DESTINATION_GAIN, (double)(-0.376287f * (float)this.getGeneratorValue(o4, 48)));
                }
                if (this.getGeneratorValue(o4, 15) != 0) {
                    this.addValue(o2, ModelDestination.DESTINATION_CHORUS, this.getGeneratorValue(o4, 15));
                }
                if (this.getGeneratorValue(o4, 16) != 0) {
                    this.addValue(o2, ModelDestination.DESTINATION_REVERB, this.getGeneratorValue(o4, 16));
                }
                if (this.getGeneratorValue(o4, 56) != 100) {
                    int generatorValue22 = this.getGeneratorValue(o4, 56);
                    if (generatorValue22 == 0) {
                        o2.getConnectionBlocks().add(new ModelConnectionBlock(null, (double)(n9 * 100), new ModelDestination(ModelDestination.DESTINATION_PITCH)));
                    }
                    else {
                        o2.getConnectionBlocks().add(new ModelConnectionBlock(null, (double)(n9 * (100 - generatorValue22)), new ModelDestination(ModelDestination.DESTINATION_PITCH)));
                    }
                    o2.getConnectionBlocks().add(new ModelConnectionBlock(new ModelSource(ModelSource.SOURCE_NOTEON_KEYNUMBER), (double)(128 * generatorValue22), new ModelDestination(ModelDestination.DESTINATION_PITCH)));
                }
                o2.getConnectionBlocks().add(new ModelConnectionBlock(new ModelSource(ModelSource.SOURCE_NOTEON_VELOCITY, new ModelTransform() {
                    @Override
                    public double transform(double n) {
                        if (n < 0.5) {
                            return 1.0 - n * 2.0;
                        }
                        return 0.0;
                    }
                }), -2400.0, new ModelDestination(ModelDestination.DESTINATION_FILTER_FREQ)));
                o2.getConnectionBlocks().add(new ModelConnectionBlock(new ModelSource(ModelSource.SOURCE_LFO2, false, true, 0), new ModelSource(new ModelIdentifier("midi_cc", "1", 0), false, false, 0), 50.0, new ModelDestination(ModelDestination.DESTINATION_PITCH)));
                if (layer.getGlobalRegion() != null) {
                    Iterator<SF2Modulator> iterator5 = layer.getGlobalRegion().getModulators().iterator();
                    while (iterator5.hasNext()) {
                        this.convertModulator(o2, (SF2Modulator)iterator5.next());
                    }
                }
                Iterator<SF2Modulator> iterator6 = sf2LayerRegion.getModulators().iterator();
                while (iterator6.hasNext()) {
                    this.convertModulator(o2, (SF2Modulator)iterator6.next());
                }
                if (globalregion != null) {
                    Iterator<SF2Modulator> iterator7 = globalregion.getModulators().iterator();
                    while (iterator7.hasNext()) {
                        this.convertModulator(o2, (SF2Modulator)iterator7.next());
                    }
                }
                Iterator<SF2Modulator> iterator8 = sf2InstrumentRegion.getModulators().iterator();
                while (iterator8.hasNext()) {
                    this.convertModulator(o2, (SF2Modulator)iterator8.next());
                }
            }
        }
        return array;
    }
    
    private void convertModulator(ModelPerformer modelPerformer, SF2Modulator sf2Modulator) {
        ModelSource convertSource = convertSource(sf2Modulator.getSourceOperator());
        ModelSource convertSource2 = convertSource(sf2Modulator.getAmountSourceOperator());
        if (convertSource == null && sf2Modulator.getSourceOperator() != 0) {
            return;
        }
        if (convertSource2 == null && sf2Modulator.getAmountSourceOperator() != 0) {
            return;
        }
        double n = (double)sf2Modulator.getAmount();
        double[] array = { 0.0 };
        ModelSource[] array2 = { null };
        array[0] = 1.0;
        ModelDestination convertDestination = convertDestination(sf2Modulator.getDestinationOperator(), array, array2);
        double n2 = n * array[0];
        if (convertDestination == null) {
            return;
        }
        if (sf2Modulator.getTransportOperator() == 2) {
            ((ModelStandardTransform)convertDestination.getTransform()).setTransform(4);
        }
        ModelConnectionBlock modelConnectionBlock = new ModelConnectionBlock(convertSource, convertSource2, n2, convertDestination);
        if (array2[0] != null) {
            modelConnectionBlock.addSource(array2[0]);
        }
        modelPerformer.getConnectionBlocks().add(modelConnectionBlock);
        return;
    }
    
    private static ModelSource convertSource(int n) {
        if (n == 0) {
            return null;
        }
        Object o = null;
        int i = n & 0x7F;
        if ((n & 0x80) != 0x0) {
            o = new ModelIdentifier("midi_cc", Integer.toString(i));
        }
        else {
            if (i == 2) {
                o = ModelSource.SOURCE_NOTEON_VELOCITY;
            }
            if (i == 3) {
                o = ModelSource.SOURCE_NOTEON_KEYNUMBER;
            }
            if (i == 10) {
                o = ModelSource.SOURCE_MIDI_POLY_PRESSURE;
            }
            if (i == 13) {
                o = ModelSource.SOURCE_MIDI_CHANNEL_PRESSURE;
            }
            if (i == 14) {
                o = ModelSource.SOURCE_MIDI_PITCH;
            }
            if (i == 16) {
                o = new ModelIdentifier("midi_rpn", "0");
            }
        }
        if (o == null) {
            return null;
        }
        ModelSource o2 = new ModelSource(o);
        ModelStandardTransform modelStandardTransform = (ModelStandardTransform)o2.getTransform();
        if ((0x100 & n) != 0x0) {
            modelStandardTransform.setDirection(true);
        }
        else {
            modelStandardTransform.setDirection(false);
        }
        if ((0x200 & n) != 0x0) {
            modelStandardTransform.setPolarity(true);
        }
        else {
            modelStandardTransform.setPolarity(false);
        }
        if ((0x400 & n) != 0x0) {
            modelStandardTransform.setTransform(1);
        }
        if ((0x800 & n) != 0x0) {
            modelStandardTransform.setTransform(2);
        }
        if ((0xC00 & n) != 0x0) {
            modelStandardTransform.setTransform(3);
            return o2;
        }
        return o2;
    }
    
    static ModelDestination convertDestination(int n, double[] array, ModelSource[] array2) {
        ModelIdentifier modelIdentifier = null;
        switch (n) {
            case 8: {
                modelIdentifier = ModelDestination.DESTINATION_FILTER_FREQ;
                break;
            }
            case 9: {
                modelIdentifier = ModelDestination.DESTINATION_FILTER_Q;
                break;
            }
            case 15: {
                modelIdentifier = ModelDestination.DESTINATION_CHORUS;
                break;
            }
            case 16: {
                modelIdentifier = ModelDestination.DESTINATION_REVERB;
                break;
            }
            case 17: {
                modelIdentifier = ModelDestination.DESTINATION_PAN;
                break;
            }
            case 21: {
                modelIdentifier = ModelDestination.DESTINATION_LFO1_DELAY;
                break;
            }
            case 22: {
                modelIdentifier = ModelDestination.DESTINATION_LFO1_FREQ;
                break;
            }
            case 23: {
                modelIdentifier = ModelDestination.DESTINATION_LFO2_DELAY;
                break;
            }
            case 24: {
                modelIdentifier = ModelDestination.DESTINATION_LFO2_FREQ;
                break;
            }
            case 25: {
                modelIdentifier = ModelDestination.DESTINATION_EG2_DELAY;
                break;
            }
            case 26: {
                modelIdentifier = ModelDestination.DESTINATION_EG2_ATTACK;
                break;
            }
            case 27: {
                modelIdentifier = ModelDestination.DESTINATION_EG2_HOLD;
                break;
            }
            case 28: {
                modelIdentifier = ModelDestination.DESTINATION_EG2_DECAY;
                break;
            }
            case 29: {
                modelIdentifier = ModelDestination.DESTINATION_EG2_SUSTAIN;
                array[0] = -1.0;
                break;
            }
            case 30: {
                modelIdentifier = ModelDestination.DESTINATION_EG2_RELEASE;
                break;
            }
            case 33: {
                modelIdentifier = ModelDestination.DESTINATION_EG1_DELAY;
                break;
            }
            case 34: {
                modelIdentifier = ModelDestination.DESTINATION_EG1_ATTACK;
                break;
            }
            case 35: {
                modelIdentifier = ModelDestination.DESTINATION_EG1_HOLD;
                break;
            }
            case 36: {
                modelIdentifier = ModelDestination.DESTINATION_EG1_DECAY;
                break;
            }
            case 37: {
                modelIdentifier = ModelDestination.DESTINATION_EG1_SUSTAIN;
                array[0] = -1.0;
                break;
            }
            case 38: {
                modelIdentifier = ModelDestination.DESTINATION_EG1_RELEASE;
                break;
            }
            case 46: {
                modelIdentifier = ModelDestination.DESTINATION_KEYNUMBER;
                break;
            }
            case 47: {
                modelIdentifier = ModelDestination.DESTINATION_VELOCITY;
                break;
            }
            case 51: {
                array[0] = 100.0;
                modelIdentifier = ModelDestination.DESTINATION_PITCH;
                break;
            }
            case 52: {
                modelIdentifier = ModelDestination.DESTINATION_PITCH;
                break;
            }
            case 48: {
                modelIdentifier = ModelDestination.DESTINATION_GAIN;
                array[0] = -0.3762870132923126;
                break;
            }
            case 6: {
                modelIdentifier = ModelDestination.DESTINATION_PITCH;
                array2[0] = new ModelSource(ModelSource.SOURCE_LFO2, false, true);
                break;
            }
            case 5: {
                modelIdentifier = ModelDestination.DESTINATION_PITCH;
                array2[0] = new ModelSource(ModelSource.SOURCE_LFO1, false, true);
                break;
            }
            case 10: {
                modelIdentifier = ModelDestination.DESTINATION_FILTER_FREQ;
                array2[0] = new ModelSource(ModelSource.SOURCE_LFO1, false, true);
                break;
            }
            case 13: {
                modelIdentifier = ModelDestination.DESTINATION_GAIN;
                array[0] = -0.3762870132923126;
                array2[0] = new ModelSource(ModelSource.SOURCE_LFO1, false, true);
                break;
            }
            case 7: {
                modelIdentifier = ModelDestination.DESTINATION_PITCH;
                array2[0] = new ModelSource(ModelSource.SOURCE_EG2, false, true);
                break;
            }
            case 11: {
                modelIdentifier = ModelDestination.DESTINATION_FILTER_FREQ;
                array2[0] = new ModelSource(ModelSource.SOURCE_EG2, false, true);
                break;
            }
        }
        if (modelIdentifier != null) {
            return new ModelDestination(modelIdentifier);
        }
        return null;
    }
    
    private void addTimecentValue(ModelPerformer modelPerformer, ModelIdentifier modelIdentifier, short n) {
        double n2;
        if (n == -12000) {
            n2 = Double.NEGATIVE_INFINITY;
        }
        else {
            n2 = (double)n;
        }
        modelPerformer.getConnectionBlocks().add(new ModelConnectionBlock(n2, new ModelDestination(modelIdentifier)));
    }
    
    private void addValue(ModelPerformer modelPerformer, ModelIdentifier modelIdentifier, short n) {
        modelPerformer.getConnectionBlocks().add(new ModelConnectionBlock((double)n, new ModelDestination(modelIdentifier)));
    }
    
    private void addValue(ModelPerformer modelPerformer, ModelIdentifier modelIdentifier, double n) {
        modelPerformer.getConnectionBlocks().add(new ModelConnectionBlock(n, new ModelDestination(modelIdentifier)));
    }
    
    private short getGeneratorValue(Map<Integer, Short> map, int n) {
        if (map.containsKey(Integer.valueOf(n))) {
            return (short)map.get(Integer.valueOf(n));
        }
        return SF2Region.getDefaultValue(n);
    }
}
